# Bookshelf 2.0beta1 an e-text reader for J2ME devices. 
# Copyright (C) 2004 Anton Krasovsky
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from java.io import File
from bookshelf.builder import MidletBuilder, ReaderPackage, PlatformPackage
from bookshelf.font import *
from config import load, config
from bookshelf.book import BookWriter
from bookshelf.jrender import *

class Bookshelf:
    def __init__(self):
        self.books = []
        self.font_fabric = FontFabric(config('FONT_DIRECTORY'))
        self.platform = PlatformPackage(config('PLATFORM_DIR') + config('PLATFORM_JAR'))
        self.reader_package = ReaderPackage(config('READER_JAR'), self.platform)
        self.platform.setBrandModel(config('PLATFORM_BRAND'), config('PLATFORM_MODEL'))

    def add_book(self, f):
        book = Book(f, self.platform, self.font_fabric)
        self.books.append(book)            
        return book

    def del_book(self, book):
        self.books.remove(book)
        
    def save(self, f):
        # TODO check if previews been generated / ie book rendered
        builder = MidletBuilder(self.reader_package, self.platform)
        for book in self.books:
            builder.add(book.book)
        # add plugins
        for plugin in config('PLUGIN_LIST'):
            builder.addPlugin(plugin)
        # always add pager
        builder.addPlugin('pager')
        builder.write(f, config('MAKE_FOLDER'))
        
class Book:
    def __init__(self, text_file, platform, font_fabric):
        self.text_file = text_file
        self.platform = platform
        self.font_fabric = font_fabric


    def format(self, title, font_name, para_start, encoding, language, para_indent, hyphenate, justify, screen_border, interline):
        fontWriter = self.platform.fontWriter
        font = self.font_fabric.loadFont(font_name)

        dimension = self.platform.canvasDimension
        line_height = (font.getHeight() + interline)
        dimension.height = (dimension.height / line_height) * line_height

        bookWriter = BookWriter(dimension)
        bookWriter.setFont(font)
        
        paragraphSource = ParagraphSource(self.text_file, encoding, font, para_start, para_indent)

        if hyphenate:
            formatter = HyphenatingFormatter()
            formatter.setHyphenator(Hyphenator("-,.", language, "-"))
        else:
            formatter = SimpleFormatter()
        
        formatter.setWidth(bookWriter.getPageSize().width)
        formatter.setSpace(2)
        if screen_border: formatter.setMargin(1)

        paginator = Paginator()
        paginator.setFormatter(formatter)
        paginator.setParagraphSource(paragraphSource)
        paginator.setInterline(interline)

        if justify:
            lineJustifier = LineJustifier();
            lineJustifier.setWidth(bookWriter.pageSize.width);
            paginator.decorator = lineJustifier
        
        paginator.write(bookWriter)
        self.book = bookWriter.getBook()
        self.book.setTitle(title)
        return self.book